
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	CAMERA.H                                                            //
//                                                                          //
//	Definition of CAMERA class.                                         //
//                    							    //
//////////////////////////////////////////////////////////////////////////////

#if !defined ( __CAMERA__ )
	#define __CAMERA__

	#include "MATH\POINT.H"
	#include "MATH\VECTOR.H"
	#include "MATH\MATRIX.H"

	class CAMERA {

		public :

			/*	Pitch roll and 
				yaw ( in radians ).	*/

			REAL Pitch, Roll, Yaw;

			/*	And a position in
				space.	*/

			POINT Location;

			/*	And a loaded matrix
				for transforming points.	*/

			MATRIX Transformation;

			/*	Constructor sets up	
				default values for camera.	*/

			CAMERA ( void );

			/*	Get camera ready to 
				apply transformations.	*/

			virtual void Ready ( void );

			/*	Set the pitch, roll and
				yaw of the camera.	*/

			void SetAngles ( REAL p, REAL r, REAL y );

			/*	Set the position of
				the camera.	*/

			void SetLocation ( REAL x, REAL y, REAL z );

			/*	Return the location
				of the camera.	*/

			POINT& GetLocation ( void );

			/*	Return the transformation
				matrix.	*/

			MATRIX& GetMatrix ( void );

			/*	Increase the roll
				of the camera.	*/

			void RollBy ( REAL x );

			/*	Increase the pitch
				of the camera.	*/

			void PitchBy ( REAL x );

			/*	Increase the
				yaw of the camera.	*/

			void YawBy ( REAL x );

			/*	Return the pitch
				roll and yaw.	*/

			void GetPitchRollYaw ( REAL *p, REAL *r, REAL *y );

			/*	Calculate the angle between a plane 
				and a camera in viewspace.	*/

			REAL GetSurfaceAngle ( POINT *p0, POINT *p1, POINT *p2 );

			/*	Return -1, 0 or 1 depending
				on the angle of the surface.	*/

			int GetOrientation ( POINT *p0, POINT *p1, POINT *p2 );

			/*	Return the colour value
				for the given surface.	*/

			PIXEL GetLightIntensity ( POINT *p0, POINT *p1, POINT *p2, PIXEL Colour );


	};
#endif