
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	MATRIX.H                                                            //
//                                                                          //
//	Definition of MATRIX class.                                         //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////
	
#if !defined ( __MATRIX__ )
	#define __MATRIX__

	#include "GLOBAL\TYPES.H"
	#include "MATH\POINT.H"
	#include "MATH\VECTOR.H"

	class MATRIX {

		public :

			/*	A simple matrix is
				a 3 by 3 array.	*/

			REAL v [4][4];

			/*	Initialise the matrix
				with empty values.	*/

			MATRIX ( void );

			/*	Initialise the matrix with the 
				concatenation of two others.	*/

			MATRIX ( MATRIX& m1, MATRIX& m2 );

			/*	Create the identity
				matrix.	*/

			void Identity ( void );

			/*	Create a pitch
				matrix.	*/

			void Pitch ( REAL p );

			/*	Create a roll
				matrix.	*/

			void Roll ( REAL r );

			/*	Create a yaw
				matrix.	*/

			void Yaw ( REAL y );

			/*	Create a translation
				matrix.	*/

			void Translate ( POINT& Point );

			/*	Make one matrix
				equal another.	*/

			inline MATRIX& operator = ( MATRIX& m );

			/*	Break the matrix out
				into 3 vectors.	*/

			void Split ( VECTOR& vRight, VECTOR& vUp, VECTOR& vNormal );

			/*	Compose a matrix from
				the component vectors.	*/

			void Compose ( VECTOR& vRight, VECTOR& vUp, VECTOR& vNormal );

			/*	Multiply the matrix by a set of points
				after translation.	*/

			void Multiply ( int n, POINT *In, POINT *Out, POINT& Move );

			/*	Multiply the matrix
				by a set of points.	*/

			void Multiply ( int n, POINT *In, POINT *Out );
	};
#endif
