
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	PLANE.CPP                                                           //
//                                                                          //
//	Implementation of PLANE class.                                      //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////
	
#include "MATH\PLANE.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Create a plane object with undefined location and normal.                //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

PLANE :: PLANE ( void )

/*	PRE	:
	POST	:
*/

{

}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Create a plane using a point on the plane and the normal to the plane.   //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

PLANE :: PLANE ( VECTOR& n, POINT& Point )

/*	PRE	:	TRUE
	POST	:	TRUE
*/

{
	Set ( n, Point );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Assign a plane using a point on the plane and the normal to the plane.   //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void PLANE :: Set ( VECTOR& n, POINT& p )

/*	PRE	:	TRUE
	POST	:	TRUE
*/

{
	/*	Set the reference
		point.	*/

	Point = p;

	/*	Set the
		normal.	*/

	Normal = n;

	/*	And store `d' rather than 
		calculate it each time.	*/

	d = -Normal.Dot ( ( VECTOR& ) Point );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Calculates the intersection of a line and a plane.                       //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

int PLANE :: Intersection ( POINT *p1, POINT *p2, POINT *Result, INTPOINT *t1, INTPOINT *t2, INTPOINT *tResult )

/*	PRE	:	TRUE
	POST	:	Returns result of intersection of line and this plane
*/

{
	REAL	t, p, q;
	VECTOR	r ( *p2 );

	r -= *p1;
	p = Normal.Dot ( ( VECTOR& ) *p1 ) + d;
	q = Normal.Dot ( ( VECTOR& ) r );
	if ( q == 0 ) {
		if ( p == 0 ) {
		
			/*	The line was in
				the plane.	*/
							
			return ( LINEINPLANE );
		} else {

			/*	Does not intersect
				at all.	*/

			return ( PARALLEL );
		}
	}

	/*	From x = x1 + ( x2 - x1 ) * t
		find the t value.	*/
		
	t = - ( p / q );
	r *= t;

	/*	Store back into
		result.	*/

	*Result = *p1;
	*Result += r;

	/*	Find out how much the
		line changed by.	*/

	VECTOR Old ( p1, p2 ), New ( p1, Result );
	REAL   Scale, lOld, lNew;

	/*	Get the lengths
		of the lines.	*/

	lOld = Old.Length ();
	lNew = New.Length ();

	/*	Find the scale
		value.	*/

	Scale = ( lNew / lOld );

	/*	Alter the texture
		co-ordinates.	*/

	tResult->x = t1->x + ( ( REAL ) ( t2->x - t1->x ) * Scale );
	tResult->y = t1->y + ( ( REAL ) ( t2->y - t1->y ) * Scale );

	/*	Intersection is in
		`Result'.	*/

	return ( INTERSECTION );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Calculates the intersection of a line and a plane.                       //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

int PLANE :: Intersection ( POINT *p1, POINT *p2, POINT *Result )

/*	PRE	:	TRUE
	POST	:	Returns result of intersection of line and this plane
*/

{
	REAL	t, p, q;
	VECTOR	r ( *p2 );

	r -= *p1;
	p = Normal.Dot ( ( VECTOR& ) *p1 ) + d;
	q = Normal.Dot ( ( VECTOR& ) r );
	if ( q == 0 ) {
		if ( p == 0 ) {
		
			/*	The line was in
				the plane.	*/
							
			return ( LINEINPLANE );
		} else {

			/*	Does not intersect
				at all.	*/

			return ( PARALLEL );
		}
	}

	/*	From x = x1 + ( x2 - x1 ) * t
		find the t value.	*/
		
	t = - ( p / q );
	r *= t;

	/*	Store back into
		result.	*/

	*Result = *p1;
	*Result += r;

	return ( INTERSECTION );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Determine which side of a plane a point lies on.                         //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

BOOL PLANE :: IsInside ( POINT *p )

/*	PRE	:
	POST	:
*/

{
	REAL   Result;
	VECTOR Temp ( Point, *p );

	/*	Use the angle of the line
		w.r.t. the plane as the
		determinant.	*/

	Result = Normal.Dot ( Temp );

	/*	And return the
		classification.	*/

	return ( ( Result < 0 ) );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Clips a polygon to this plane.                                           //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

BOOL PLANE :: Clip ( int *nVerts, POINT *In, POINT *Out, INTPOINT *InTex, INTPOINT *OutTex )

/*	PRE	:
	POST	:
*/

{
	int		i, Next;
	BOOL		cIsIn, nIsIn;
	POINT		*cIn = In, *cOut = Out;
	INTPOINT	*tIn = InTex, *tOut = OutTex;

	/*	If x < z, then point
		is outside plane.	*/

	cIsIn = IsInside ( &cIn [0] );
	if ( ( InTex != NULL ) && ( OutTex != NULL ) ) {
		for ( i = 0; i < *nVerts; i++ ) {

			/*	Keep first vertex if
				it's inside plane.	*/

			if ( cIsIn == TRUE ) {
				*cOut++ = *cIn;
				*tOut++ = *tIn;
			}

			/*	Get the next
				vertex.	*/

			Next = ( i + 1 ) % *nVerts;

			/*	Classify with respect
				to the plane.	*/

			nIsIn = IsInside ( &In [Next] );

			/*	If either is outside the
				plane, add clipped vertex.	*/

			if ( nIsIn != cIsIn ) {
				Intersection ( cIn, &In [Next], cOut++, tIn, &InTex [Next], tOut++ );
			}

			/*	Make the next result
				the current result.	*/

			cIsIn = nIsIn;

			/*	And go to the next
				input vertex.	*/

			cIn++;
			tIn++;
		}
	} else {

		/*	Clipping without
			texture clipping.	*/

		for ( i = 0; i < *nVerts; i++ ) {

			/*	Keep first vertex if
				it's inside plane.	*/

			if ( cIsIn == TRUE ) {
				*cOut++ = *cIn;
			}

			/*	Get the next
				vertex.	*/

			Next = ( i + 1 ) % *nVerts;

			/*	Classify with respect
				to the plane.	*/

			nIsIn = IsInside ( &In [Next] );

			/*	If either is outside the
				plane, add clipped vertex.	*/

			if ( nIsIn != cIsIn ) {
				Intersection ( cIn, &In [Next], cOut++ );
			}

			/*	Make the next result
				the current result.	*/

			cIsIn = nIsIn;

			/*	And go to the next
				input vertex.	*/

			cIn++;
		}
	}

	/*	Find the total number 
		of emitted vertices.	*/

	*nVerts = ( int) ( cOut - Out );

	/*	And return the
		result of clipping.	*/

	if ( *nVerts < 3 ) {
		return ( FALSE );
	} else {	
		return ( TRUE );
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Projects a polygon from a light source onto a plane.                     //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void PLANE :: Project ( POINT *Light, int nVerts, POINT *In, POINT *Out )

/*	PRE	:	This only correct if light is above input points
	POST	:	Out contains projection of In from Light
*/

{
	int i;

	for ( i = 0; i < nVerts; i++ ) {
		Intersection ( Light, &In [i], &Out [i], NULL, NULL, NULL );
	}
}

