
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	PLANE.H                                                             //
//                                                                          //
//	Definition of PLANE class.                                          //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////
	
#if !defined ( __PLANE__ )
	#define __PLANE__

	#include "MATH\VECTOR.H"

	class PLANE {

		public :
			/*	Some values to describe how a 
				line is oriented to a plane.	*/

			enum { LINEINPLANE, PARALLEL, INTERSECTION };

			/*	All planes are described
				by a plane normal.....	*/

			VECTOR 	Normal;

			/*	and a reference
				point.	*/

			POINT Point;

			/*	The `d' value ( from ax+bx+cx+d=0 can	
				be pre-calculated for speed.	*/

			REAL	d;

			/*	Operation to create
				a null plane.	*/

			PLANE ( void );

			/*	Operation to create 	
				a plane.	*/
				
			PLANE ( VECTOR& n, POINT& Point );

			/*	And setting the 
				plane.	*/

			void Set ( VECTOR& n, POINT& Point );

			/*	Operation to find the intersection
				between a line and a plane.	*/

			int Intersection ( POINT *p1, POINT *p2, POINT *Result );

			/*	Determine which side of
				the plane a point is on.	*/

			BOOL IsInside ( POINT *p );

			/*	Clip a polygon
				to the plane.	*/

			int Intersection ( POINT *p1, POINT *p2, POINT *Result, INTPOINT *t1, INTPOINT *t2, INTPOINT *tResult );

			/*	Clip a textured
				polygon to a plane.	*/

			BOOL Clip ( int *nVerts, POINT *In, POINT *Out, INTPOINT *InTex, INTPOINT *OutTex );

			/*	Project a polygon onto a plane
				( shadow attenuation ).	*/

			void Project ( POINT *Light, int nVerts, POINT *In, POINT *Out );

	};
#endif
