
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	VECTOR.CPP                                                          //
//                                                                          //
//	Definition of VECTOR class.                                         //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////
	
#if !defined ( __VECTOR__ )
	#define __VECTOR__

	#include "MATH\POINT.H"

	class VECTOR : public POINT {

		public :

			/*	default VECTOR initialiser
				does nout.	*/

			VECTOR ( void );

			/*	Initiliase with 3 REAL
				values.	*/

			VECTOR ( REAL x, REAL y, REAL z );

			/*	Initialise a vector
				with another vector.	*/

			VECTOR ( VECTOR& Vector );

			/*	Initialise a vector
				with a point.	*/

			VECTOR ( POINT& Point );

			/*	Create a vector from
				2 points.	*/

			VECTOR ( POINT& From, POINT& To );

			/*	Create a vector from
				2 points.	*/

			VECTOR ( POINT *From, POINT *To );

			/*	Set a vector to some
				initial values.	*/

			void Set ( REAL x, REAL y, REAL z );

			/*	Take the cross product 
				of this vector and another.	*/

			void Cross ( VECTOR& v );

			/*	Take the dot product of this
				vector and another.	*/

			REAL Dot ( VECTOR& v );

			/*	Find the length of
				this vector.	*/

			REAL Length ( void );

			/*	Make this vector of
				unit length.	*/

			void Unit ( void );

			/*	Make a vector from
				2 points.	*/

			void Make ( POINT *From, POINT *To );

			/*	Assign a point
				to a vector.	*/

			VECTOR& operator = ( POINT& Point );

			/*	Rotate vector in
				XZ plane.	*/

			void RotateXZ ( REAL Angle );

			/*	Rotate a vector in
				YZ plane.	*/

			void RotateYZ ( REAL Angle );
	};
#endif





