	
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	MODE.H                                                              //
//                                                                          //
//	Definition of MODE class.                                           //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////
	
#if !defined ( __GRAPHICSMODE__ )
	#define  __GRAPHICSMODE__

	#include "GLOBAL\TYPES.H"
	#include "MODE\MODEX.H"
	#include "MATH\POINT.H"
	#include "VIEW\WINDOW.H"
	#include "COLOUR\COLOUR.H"
	#include "TEXTURE\TEXTURE.H"

	enum MODE {
		_320by400in256
	};

	class GRAPHICSMODE {

		public :

				/*	Store the current
					mode we are in.	*/

			MODE Mode;

				/*	The current window
					to work with.	*/

			int MinX, MinY, MaxX, MaxY;

				/*	The current texture
					map to work with.	*/

			TEXTUREMAP *Texture;
			BOOL Transparent;


				/*	Page number and start
					offset for VGA.	*/

			unsigned Offset, Page;

				/*	Width and length
					of the page.	*/

			int Width, Length;

				/*	and a fast lookup
					scan table.	*/

			int *ScanTable, *Maxima, *Minima;

				/*	The structure for a
					texture map edge.	*/

			typedef struct {
				int	Direction;
				int	RemainingScans;
				int	CurrentEnd;
				long	SourceX;
				long	SourceY;
				long	SourceStepX;
				long	SourceStepY;
				int	DestX;
				int	DestXIntStep;
				int	DestXDirection;
				int	DestXErrTerm;
				int	DestXAdjUp;
				int	DestXAdjDown;
			} EDGESCAN;

				/*      Output to
					an index register.	*/

			void TOINDEXREG ( int r, int v, int i ) {
				outport ( r, ( v << 8 ) + i );
			}

				/*      Set the VGA
					map mask.	*/

			void MAPMASK ( int a ) {
				TOINDEXREG ( SC_INDEX, a, MAP_MASK );
			}

				/*	Set the screen start
					address high.	*/

			void STARTHI ( int a ) {
				TOINDEXREG ( CRTC_INDEX, a, START_ADDRESS_HIGH );
			}

				/*	Set the screen start
					address low.	*/

			void STARTLO ( int a ) {
				TOINDEXREG ( CRTC_INDEX, a, START_ADDRESS_LOW );
			}

				/*	Set the screen
					start address.	*/

			void START ( unsigned a ) {
				STARTHI ( ( a >> 8 ) ); STARTLO ( ( a & 0xFF ) );
			}

				/*	Set the current
					VGA page.	*/

			void PAGE ( int a ) {
				STARTLO ( 0 ); STARTHI ( ( a * 0x80 ) );
			}

				/*	Return the screen
					address of an x,y.	*/

			PIXEL * GETADDRESS ( int x, int y ) {
				return ( ( PIXEL * ) ( 0xA0000000L + Offset + ( x >> 2 ) + ScanTable [y] ) );
			}

				/*	Return the address for
					any page ( minus the offset ).	*/

			PIXEL * GETPAGEADDRESS ( int x, int y ) {
				return ( ( PIXEL * ) ( 0xA0000000L + ( x >> 2 ) + ScanTable [y] ) );
			}

				/*	Set the map mask
					page given an x.	*/

			void SELECTPAGE ( int x ) {
				int Masks [] = { 1, 2, 4, 8 };
				MAPMASK ( Masks [x & 3] );
			}

				/*	Place a pixel onto
					the screen.	*/

			void PUTPIXEL ( int x, int y, int Colour ) {
				SELECTPAGE ( x );
				* ( GETADDRESS ( x, y ) ) = Colour;
			}

				/*	Step the edge of
					a textured polygon.	*/

			int TexStepEdge	( EDGESCAN * );

				/*	Setup the edge of a
					textured polygon.	*/

			int TexSetUpEdge ( EDGESCAN *, int );

				/*	Scan out a line from
					a textured polygon.	*/

			void TexScanOutLine ( EDGESCAN *, EDGESCAN * );

				/*	Operation to overscan
					a colour onto the screen.	*/

			void OverScan ( int x1, int x2, int y, int Colour );

				/*	Scan out the edge of
					a polygon.	*/

			void Scan ( INTPOINT *a, INTPOINT *b, int *Buff );

				/*	Fast copy from one part
					of VGA to another.	*/

			void Blitter ( int dx, int dy, int dl, int dt, int dr, int db );

				/*	Open the mode using
					bios calls.	*/

			GRAPHICSMODE ( MODE ThisMode );

				/*	Close the mode
					down again.	*/

			~GRAPHICSMODE ();

				/*	Flip the graphics
					page from 0 to 1.	*/

			void FlipPage ( void );

				/*	Render a flat shaded
					polygon onto the page.	*/

			void FlatShadePolygon ( int nVertices, INTPOINT *Vertex, PIXEL Colour );

				/*	Render a textured
					polygon onto the page.	*/

			void TextureMapPolygon ( int nVertices, INTPOINT *Vertex, INTPOINT *TexVerts, BOOL Culled, PIXEL Colour );

				/*	Render a wu-line
					( anti-aliased ).	*/

			void WuLine ( int X0, int Y0, int X1, int Y1, COLOUR *Colour );

				/*	Clear the mode to a
					fixed colour.	*/

			void Clear ( PIXEL StartColour );

				/*	Operation to render
					a horizontal line.	*/

			void hLine ( int x1, int x2, int y, PIXEL Colour );

				/*	Operation to render
					a vertical line.	*/

			void vLine ( int x, int y1, int y2, PIXEL Colour );

				/*	Operation to blitter
					a rectangle to the screen.	*/

			void Rectangle ( int l, int t, int r, int b, PIXEL Colour );

				/*	Blitter some text
					to the screen.	*/

			void OutText ( int x, int y, char *String, PIXEL Foreground, PIXEL Background );

				/*	And do the text
					without background.	*/

			void OutText ( int x, int y, char *String, PIXEL Foreground );

				/*	Set the current
					texture map.	*/

			void SetTexture ( TEXTUREMAP *Texture, BOOL Transparent );

				/*	Set the current
					window.	*/

			void SetWindow ( WINDOW *Window );                	
	};
#endif
