
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	MODEPOLY.CPP                                                        //
//                                                                          //
//	Implementation of MODE class.                                       //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////
	
#include "MODE\MODE.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Operation to render a flat shaded polygon to the screen.                 //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void GRAPHICSMODE :: FlatShadePolygon ( int nVertices, INTPOINT *Vertex, PIXEL Colour )

/*	PRE	:	TRUE
	POST	:	Polygon is rendered to the screen.
*/

{
	int	i, Next, TopVertex, BottomVertex, LeftVertex, RightVertex;
	int	Left = 32767, Right = -32767, Top = 32767, Bottom = -32767;

	/*      Find the top, bottom, left
		and rightmost vertices.        */

	for ( i = 0; i < nVertices; i++ ) {
		if ( Vertex [i].y < Top ) {
			Top = Vertex [i].y;
			TopVertex = i;
		}
		if ( Vertex [i].y > Bottom ) {
			Bottom = Vertex [i].y;
			BottomVertex = i;
		}
		if ( Vertex [i].x < Left ) {
			Left = Vertex [i].x;
		}
		if ( Vertex [i].x > Right ) {
			Right = Vertex [i].x;
		}
	}

	/*	Reject if the polygon
		is obviously off screen.	*/

	if ( Bottom < MinY || Top > MaxY || Left > MaxX || Right < MinX ) {
		return;
	}

	/*	From the top vertex, move right
		scanning out the rightmost edges.	*/

	RightVertex = TopVertex;
	do {
		/*	Get the next
			vertex to this.	*/

		Next = ( RightVertex + 1 );
		if ( Next == nVertices ) {
			Next = 0;
		}

		/*	Scan convert the current
			vertex to the next one.	*/

		Scan ( &Vertex [RightVertex], &Vertex [Next], Maxima );

		/*	Go to the next
			vertex.	*/

		RightVertex++;
		if ( RightVertex == nVertices ) {
			RightVertex = 0;
		}
	} while ( RightVertex != BottomVertex );

	/*	From the top vertex, move left
		scanning out the leftmost edges.	*/

	LeftVertex = TopVertex;
	do {
		/*	Get the next
			vertex to this.	*/

		Next = ( LeftVertex - 1 );
		if ( Next < 0 ) {
			Next = ( nVertices - 1 );
		}

		/*	Scan convert the current
			vertex to the next one.	*/

		Scan ( &Vertex [LeftVertex], &Vertex [Next], Minima );

		/*	Go to the next
			vertex.	*/

		LeftVertex = ( LeftVertex - 1 );
		if ( LeftVertex < 0 ) {
			LeftVertex = ( nVertices - 1 );
		}
	} while ( LeftVertex != BottomVertex );
  
	/*	Now blitter the
		scan bucket.	*/

	{
		int	i;

		/*	Clip to top and
			bottom of screen.	*/

		if ( Top < MinY )
			Top = MinY;
		if ( Bottom > MaxY )
			Bottom = MaxY;

		for ( i = Top; i <= Bottom; i++ ) {

			/*	If they are the wrong
				way around, swap them.	*/

			if ( Minima [i] > Maxima [i] ) {
				SWAP ( Minima [i], Maxima [i] );
			}

			/*	Ignore if span
				is off screen.	*/

			if ( ( Minima [i] > MaxX ) || ( Maxima [i] < MinX ) ) {
				continue;
			}

			/*	Clip this span
				to the screen.	*/

			if ( Minima [i] < MinX ) 
				Minima [i] = MinX;
			if ( Maxima [i] > MaxX )
				Maxima [i] = MaxX;

			/*	Blitter the
				line.	*/

			OverScan ( Minima [i], Maxima [i], i, Colour );
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Auxhillary DDA algorithm to trace out the egde of a line segment.        //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void GRAPHICSMODE :: Scan ( INTPOINT *a, INTPOINT *b, int *Buff )

/*	PRE	:	
	POST	:
*/

{
	int d, x, y, ax, ay, sx, sy, dx, dy;

	/*	Setup the DDA
		algorithm values.	*/

	dx = ( b->x - a->x );
	dy = ( b->y - a->y );
	ax = ABS ( dx ) << 1;
	ay = ABS ( dy ) << 1;
	sx = SGN ( dx );
	sy = SGN ( dy );

	x = a->x;
	y = a->y;

	if ( ax > ay ) {

		/*	x dominant
			line segment.	*/

		d = ay - ( ax >> 1 );
		for ( ;; ) {

			/*	Store the x value
				into the bucket.	*/

			if ( y >= 0 && y < Length ) {
				Buff [y] = x;
			}

			/*	Terminate
				yet?	*/

			if ( x == b->x ) {
				return;
			}

			/*	Calculate y
				step down line.	*/

			if ( d >= 0 ) {
				y += sy;
				d -= ax;
			}

			/*	Step x by
				one pixel.	*/

			x += sx;
			d += ay;
		}
	} else {

		/*	y dominant
			line segment.	*/

		d = ax - ( ay >> 1 );
		for ( ;; ) {

			/*	Store the x value
				into the bucket.	*/

			if ( y >= 0 && y < Length ) {
				Buff [y] = x;
			}
			
			/*	Terminate
				yet?	*/

			if ( y == b->y )
				return;

			/*	Calculate x
				step along.	*/

			if ( d >= 0 ) {
				x += sx;
				d -= ay;
			}

			/*	Step y by
				one pixel.	*/

			y += sy;
			d += ax;
		}
	}
}

