
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	MODESET.CPP                                                         //
//                                                                          //
//	Implementation of MODE class.                                       //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////
	
#include "MODE\MODE.H"

#include <dos.h>
#include <mem.h>
#include <alloc.h>

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Call the BIOS to setup the given graphics mode ( only 1 supported ).     //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

GRAPHICSMODE :: GRAPHICSMODE ( MODE ThisMode )

/*	PRE	:	TRUE
	POST	:	Graphics mode `mode' is initialised.
*/

{
	/*	Set the starting
		page number and offset.	*/

	Page = 1;
	Offset = 0x8000;
	Mode = ThisMode;

	switch ( ThisMode ) {

		case _320by400in256 :

			{
				struct		REGPACK Regs;
				unsigned char	Byte;

				/*	Set the graphics
					mode to VGA 13h	*/

				Regs.r_ax = 0x13;
				intr ( 0x10, &Regs );

				/*      Change CPU addressing
					mode.	*/

				outportb ( SC_INDEX, MEMORY_MODE );
				Byte = ( ( inportb ( SC_INDEX + 1 ) & ~0x8 ) | 0x4 );
				outportb ( SC_INDEX + 1, Byte );
				outportb ( GC_INDEX, GRAPHICS_MODE );
				Byte = ( inportb ( GC_INDEX + 1 ) & ~0x1 );
				outportb ( GC_INDEX + 1, Byte );
				outportb ( GC_INDEX, MISCELLANEOUS );
				Byte = ( inportb ( GC_INDEX + 1 ) & ~0x2 );
				outportb ( GC_INDEX + 1, Byte );

				/*	Clear the screen,
					so no garabage shown.	*/

				MAPMASK ( 0xF );
				setmem ( ( unsigned char * ) 0xA0000000L, ( unsigned ) 64000L, 0 );

				/*	Tweak the mode by not
					scanning each line twice.	*/

				outportb ( CRTC_INDEX, MAX_SCAN_LINE );
				Byte = ( inportb ( CRTC_INDEX + 1 ) & ~0x1F );
				outportb ( CRTC_INDEX + 1, Byte );

				/*	Change from doubleword to
					byte CRTC scanning mode.	*/

				outportb ( CRTC_INDEX, UNDERLINE );
				Byte = ( inportb ( CRTC_INDEX + 1 ) & ~0x40 );
				outportb ( CRTC_INDEX + 1, Byte );
				outportb ( CRTC_INDEX, MODE_CONTROL );
				Byte = ( inportb ( CRTC_INDEX + 1 ) | 0x40 );
				outportb ( CRTC_INDEX + 1, Byte );

				/*	Set the width
					and length.	*/

				Width = 320;
				Length = 400;
			}
	}

	/*	Now setup the scan
		table.	*/

	ScanTable = ( int * ) malloc ( sizeof ( int ) * Length );

	/*	Width is 80 bytes for all of
		these planar modes.	*/

	int i;
	for ( i = 0; i < Length; i++ ) {
		ScanTable [i] = i * 80;
	}

	/*	and the maxima and
		minima buckets.	*/

	Maxima = ( int * ) malloc ( sizeof ( int ) * Length );
	Minima = ( int * ) malloc ( sizeof ( int ) * Length );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Destroy the mode by returning the to the previous mode.                  //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

GRAPHICSMODE :: ~GRAPHICSMODE ()

{
}

