
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	BIRD.CP                                                             //
//                                                                          //
//	Implementation of BIRD class.                                       //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////
	
#include "OBJECT\BIRD.H"

#include <stdlib.h>

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Constructor sets up the bird orientation and position.                   //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

BIRD :: BIRD ( void )

/*	PRE	:
	POST	:
*/

{
	static TEXTUREMAP Scales ( ".\\SCALES.PCX" );
	static TEXTUREMAP Head	 ( ".\\HEAD.PCX" );

	/*	Setup the
		vertices.	*/

	Vertices [0].Set ( -20.0, 0.0, 35.0 );
	Vertices [1].Set ( -20.0, 0.0, -35.0 );
	Vertices [2].Set ( -40.0, 0.0, 30.0 );
	Vertices [3].Set ( -40.0, 0.0, -30.0 );
	Vertices [4].Set ( -100.0, 0.0, 25.0 );
	Vertices [5].Set ( -100.0, 0.0, -25.0 );
	Vertices [6].Set ( -140.0, 0.0, 20.0 );
	Vertices [7].Set ( -140.0, 0.0, -20.0 );
	Vertices [8].Set ( -200.0, 0.0, 10.0 );
	Vertices [9].Set ( -200.0, 0.0, -10.0 );
	Vertices [10].Set ( -210.0, 0.0, 0.0 );

	Vertices [11].Set ( 20.0, 0.0, 35.0 );
	Vertices [12].Set ( 20.0, 0.0, -35.0 );
	Vertices [13].Set ( 40.0, 0.0, 30.0 );
	Vertices [14].Set ( 40.0, 0.0, -30.0 );
	Vertices [15].Set ( 100.0, 0.0, 25.0 );
	Vertices [16].Set ( 100.0, 0.0, -25.0 );
	Vertices [17].Set ( 140.0, 0.0, 20.0 );
	Vertices [18].Set ( 140.0, 0.0, -20.0 );
	Vertices [19].Set ( 200.0, 0.0, 10.0 );
	Vertices [20].Set ( 200.0, 0.0, -10.0 );
	Vertices [21].Set ( 210.0, 0.0, 0.0 );

	Vertices [22].Set ( 0.0, 20.0, 20.0 );
	Vertices [23].Set ( 0.0, 0.0, 70.0 );

	/*	And the references to the
		vertices make polygons.	*/

	static int	lw1 [] = { 0, 1, 3, 2 },
			lw2 [] = { 2, 3, 5, 4 },
			lw3 [] = { 4, 5, 7, 6 },
			lw4 [] = { 6, 7, 9, 8 },
			lw5 [] = { 8, 9, 10 };

	static int	rw1 [] = { 13, 14, 12, 11 },
			rw2 [] = { 15, 16, 14, 13 },
			rw3 [] = { 17, 18, 16, 15 },
			rw4 [] = { 19, 20, 18, 17 },
			rw5 [] = { 20, 19, 21 };

	static int	Backl [] = { 22, 1, 0 },
			Backr [] = { 12, 11, 22 };

	static int	Neckl [] = { 0, 23, 22 },
			Neckr [] = { 22, 23, 11 };

	/*	Some texture space
		co-ordinates.	*/

	static INTPOINT TexCords [] = {
		{ 0, 0 }, { 127, 0 }, { 127, 127 }, { 0, 127 }
	};

	/*	and the polygon
		structures.	*/

	Polygons [0].Set  ( NOCULL | TEXTURED, 4, lw1 );
	Polygons [1].Set  ( NOCULL | TEXTURED, 4, lw2 );
	Polygons [2].Set  ( NOCULL | TEXTURED, 4, lw3 );
	Polygons [3].Set  ( NOCULL | TEXTURED, 4, lw4 );
	Polygons [4].Set  ( NOCULL | TEXTURED, 3, lw5 );
	Polygons [5].Set  ( NOCULL | TEXTURED, 4, rw1 );
	Polygons [6].Set  ( NOCULL | TEXTURED, 4, rw2 );
	Polygons [7].Set  ( NOCULL | TEXTURED, 4, rw3 );
	Polygons [8].Set  ( NOCULL | TEXTURED, 4, rw4 );
	Polygons [9].Set  ( NOCULL | TEXTURED, 3, rw5 );
	Polygons [10].Set ( NOCULL | TEXTURED, 3, Backl );
	Polygons [11].Set ( NOCULL | TEXTURED, 3, Backr );
	Polygons [12].Set ( NOCULL | TEXTURED, 3, Neckl );
	Polygons [13].Set ( NOCULL | TEXTURED, 3, Neckr );

	/*	And set the colours	
		for the surfaces.	*/

	Polygons [0].Set	( INSIDE | OUTSIDE, 80, &Scales, TexCords );
	Polygons [1].Set	( INSIDE | OUTSIDE, 80, &Scales, TexCords );
	Polygons [2].Set	( INSIDE | OUTSIDE, 80, &Scales, TexCords );
	Polygons [3].Set	( INSIDE | OUTSIDE, 80, &Scales, TexCords );
	Polygons [4].Set	( INSIDE | OUTSIDE, 80, &Scales, TexCords );
	Polygons [5].Set	( INSIDE | OUTSIDE, 80, &Scales, TexCords );
	Polygons [6].Set	( INSIDE | OUTSIDE, 80, &Scales, TexCords );
	Polygons [7].Set	( INSIDE | OUTSIDE, 80, &Scales, TexCords );
	Polygons [8].Set	( INSIDE | OUTSIDE, 80, &Scales, TexCords );
	Polygons [9].Set	( INSIDE | OUTSIDE, 80, &Scales, TexCords );
	Polygons [10].Set	( INSIDE | OUTSIDE, 80, &Head,   TexCords );
	Polygons [11].Set	( INSIDE | OUTSIDE, 80, &Head,   TexCords );
	Polygons [12].Set	( INSIDE | OUTSIDE, 80, &Head,   TexCords );
	Polygons [13].Set	( INSIDE | OUTSIDE, 80, &Head,   TexCords );

	/*	Set initial orientation
		of the bird ( towards camera ).	*/

	MyCamera.SetAngles ( 0.0, 0.0, 180.0 );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Function to set the bird up for projection and call the projector.       //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void BIRD :: Show ( VIEWPROJECTION& View )

/*	PRE	:
	POST	:
*/

{
	static REAL	Angles [5] = { 0.0, 0.0, 0.0, 0.0, 0.0 }, Inc = 1.0, Up;
	POINT		World [24], Dummy ( 0.0, 0.0, 0.0 );
	static		BOOL Turning = FALSE, Flapping = TRUE;
	int		i;

	/*	Put the vertices into	
		temporary workspace.	*/

	for ( i = 0; i < 24; i++ ) {
		World [i] = Vertices [i];
	}

	/*	Now rotate the segments
		by the given angles.	*/

	RotateWingSegment ( Angles [0], &World [2], &World [3] );
	RotateWingSegment ( Angles [1], &World [4], &World [5] );
	RotateWingSegment ( Angles [2], &World [6], &World [7] );	
	RotateWingSegment ( Angles [3], &World [8], &World [9] );	
	RotateWingSegment ( Angles [4], &World [10], &Dummy );

	RotateWingSegment ( -Angles [0], &World [13], &World [14] );
	RotateWingSegment ( -Angles [1], &World [15], &World [16] );
	RotateWingSegment ( -Angles [2], &World [17], &World [18] );	
	RotateWingSegment ( -Angles [3], &World [19], &World [20] );	
	RotateWingSegment ( -Angles [4], &World [21], &Dummy );

	if ( Flapping == TRUE ) {

		/*	Increase\decrease
			the wing angles.	*/

		for ( i = 0; i < 4; i++ ) {
			Angles [i] += ( Inc * ( REAL ) ( ( i + 1 ) << 1 ) );
		}

		Angles [4] += ( Inc * ( REAL ) ( i << 1 ) );

		/*	And flap the other way
			if we reach the end.	*/

		if ( Angles [0] > 16.0 || Angles [0] < -16.0 ) {
			Inc = -Inc;
		}
	}

	/*	Now setup the local	
		camera, etc.	*/

	POINT&	Location = MyCamera.GetLocation ();

	/*	Move up and
		down.	*/

	Location.v [1] += ( Inc * 2 );

	/*	Get the camera
		ready.	*/

	MyCamera.Ready ();

	/*	Transform the vertices
		locally.	*/

	MyCamera.GetMatrix ().Multiply ( 24, World, World );

	/*	Translate the object
		into the world.	*/

	for ( i = 0; i < 24; i++ ) {
		World [i] += Location;
	}

	/*	Contain information
		in a GEOMETRIC structure.	*/
	
	static GEOMETRY MyGeometry ( 24, World, 14, Polygons );

	/*	And pass them on
		to the transformation.	*/

	View += MyGeometry;

	/*	Move in the direction
		of the rotated normal.	*/

	POINT Normal ( 0.0, 0.0, 20.0 );

	MyCamera.GetMatrix ().Multiply ( 1, &Normal, &Normal );
	Location += Normal;
	if ( Location.v [1] < 250.0 ) {
		Location.v [1] = 250.0;
	}

	/*	Turn myself if moving away from
		the center point.	*/

	VECTOR Current ( MyCamera.GetLocation () );
	static REAL MyRoll = 0.0, MyYaw = 0.0;

	if ( Turning == FALSE ) {
		if ( Current.Length () > 2000.0 ) {
			Turning = TRUE;
			Flapping = FALSE;
		}
		if ( MyYaw > 0.0 ) {
			MyYaw -= 4;
			MyCamera.RollBy ( 2.0 );
		} else {
			Flapping = TRUE;
		}
	}

	if ( Turning == TRUE ) {
		if ( MyYaw < 180.0 ) {
			MyYaw += 4;
			MyCamera.YawBy ( 4.0 );
			MyCamera.RollBy ( -2.0 );
		} else {
			Turning = FALSE;
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Function to set the bird up for projection and call the projector.       //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void BIRD :: RotateWingSegment ( REAL Angle, POINT *v1, POINT *v2 )

/*	PRE	:	TRUE
	POST	:	The given vertices are rotated
*/

{
	REAL Radians = RADIAN ( Angle ), x, y, Cos, Sin;

	Cos = cos ( Radians );
	Sin = sin ( Radians );

	/*	Rotate 
		explicitly.	*/

	x = v1->v [0] * Cos - v1->v [1] * Sin;
	y = v1->v [0] * Sin + v1->v [1] * Cos;
	v1->v [0] = x;
	v1->v [1] = y;
	x = v2->v [0] * Cos - v2->v [1] * Sin;
	y = v2->v [0] * Sin + v2->v [1] * Cos;
	v2->v [0] = x;
	v2->v [1] = y;
}

