
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	CUBE.CP                                                             //
//                                                                          //
//	Implementation of CUBE class.                                       //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////
	
#include "OBJECT\CUBE.H"

#include <stdlib.h>

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Constructor sets up the bird orientation and position.                   //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

CUBE :: CUBE ( char *Path )

/*	PRE	:
	POST	:
*/

{
	Texture.Initialise ( Path );

	/*	Setup the
		vertices.	*/

	Vertices [0].Set ( -20.0, 20.0, -20.0 );
	Vertices [1].Set ( 20.0, 20.0, -20.0 );
	Vertices [2].Set ( 20.0, -20.0, -20.0 );
	Vertices [3].Set ( -20.0, -20.0, -20.0 );
	Vertices [4].Set ( -20.0, 20.0, 20.0 );
	Vertices [5].Set ( 20.0, 20.0, 20.0 );
	Vertices [6].Set ( 20.0, -20.0, 20.0 );
	Vertices [7].Set ( -20.0, -20.0, 20.0 );

	/*	And the references to the
		vertices make polygons.	*/

	static int	f [] = { 3, 2, 1, 0 },
			b [] = { 6, 7, 4, 5 },
			l [] = { 0, 4, 7, 3 },
			r [] = { 2, 6, 5, 1 },
			t [] = { 5, 4, 0, 1 },
			e [] = { 7, 6, 2, 3 };

	/*	Some texture space
		co-ordinates.	*/

	static INTPOINT TexCords [] = {
		{ 0, 0 }, { 127, 0 }, { 127, 127 }, { 0, 127 }
	};

	/*	and the polygon
		structures.	*/

	Polygons [0].Set  ( CULL | TEXTURED, 4, f );
	Polygons [1].Set  ( CULL | TEXTURED, 4, b );
	Polygons [2].Set  ( CULL | TEXTURED, 4, t );
	Polygons [3].Set  ( CULL | TEXTURED, 4, e );
	Polygons [4].Set  ( CULL | TEXTURED, 4, r );
	Polygons [5].Set  ( CULL | TEXTURED, 4, l );

	/*	And set the colours	
		for the surfaces.	*/

	Polygons [0].Set ( OUTSIDE, 48, &Texture, TexCords );
	Polygons [1].Set ( OUTSIDE, 48, &Texture, TexCords );
	Polygons [2].Set ( OUTSIDE, 48, &Texture, TexCords );
	Polygons [3].Set ( OUTSIDE, 48, &Texture, TexCords );
	Polygons [4].Set ( OUTSIDE, 48, &Texture, TexCords );
	Polygons [5].Set ( OUTSIDE, 48, &Texture, TexCords );

	/*	Set initial orientation
		of the bird ( towards camera ).	*/

	MyCamera.SetAngles ( 0.0, 0.0, 0.0 );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Function to set the bird up for projection and call the projector.       //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void CUBE :: Show ( VIEWPROJECTION& View )

/*	PRE	:
	POST	:
*/

{
	POINT		World [8];
	int		i;

	/*	Put the vertices into	
		temporary workspace.	*/

	for ( i = 0; i < 8; i++ ) {
		World [i] = Vertices [i];
	}

	/*	Now setup the local
		camera, etc.	*/

	POINT&	Location = MyCamera.GetLocation ();

	/*	Get the camera
		ready.	*/

	MyCamera.PitchBy ( 5.0 );
	MyCamera.RollBy ( 5.0 );
	MyCamera.YawBy ( 5.0 );

	MyCamera.Ready ();

	/*	Transform the vertices
		locally.	*/

	MyCamera.GetMatrix ().Multiply ( 8, World, World );

	/*	Translate the object
		into the world.	*/

	for ( i = 0; i < 8; i++ ) {
		World [i] += Location;
	}

	/*	Contain information
		in a GEOMETRIC structure.	*/
	
	static GEOMETRY MyGeometry ( 8, World, 6, Polygons );

	/*	And pass them on
		to the transformation.	*/

	View += MyGeometry;
}


