
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	GROUND.CPP                                                          //
//                                                                          //
//	Implementation of GROUND class.                                     //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

#include "OBJECT\GROUND.H"

#include <stdlib.h>
#include <alloc.h>

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Setup the ground plane ( constructor ).                                  //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

GROUND :: GROUND ( void )	

{
	/*	Set the ground
		plane vertices.	*/

	int	i, j;
	REAL	x, z;

	/*	Set to random
		values.	*/

	for ( i = 0, x = -3000; i < GRIDSIZE; i++, x += ( 6000.0 / GRIDSIZE ) ) {
		for ( j = 0, z = -3000; j < GRIDSIZE; j++, z += ( 6000.0 / GRIDSIZE ) ) {
			Vertices [i][j].Set ( x, ( REAL ) random ( 200 ), z );
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Show the plane ( directly to the buffer ) after transform and clip.      //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void GROUND :: Show ( VIEWPROJECTION& View, GRAPHICSMODE& Buffer )

/*	PRE	:	TRUE
	POST	:	Ground display's itself
*/

{
	static	TEXTUREMAP 	MyTexture ( ".\\head.pcx" );
	static	POINT		ViewVerts [GRIDSIZE*GRIDSIZE], InPoly [4], OutPoly [8];
	static	INTPOINT 	IntPoly [8], InTex [] = { { 0, 0 }, { 127, 0 }, { 127, 127 }, { 0, 127 } }, OutTex [8];

	CAMERA	*ViewCam = View.GetCamera (), *cLight = View.GetLightSource ();
	POINT	MoveBy;
	POINT&	Location = ViewCam->GetLocation ();
	int	i, j, nVerts;
	PIXEL	Colour;

	/*	Extract the heightfield
		about the observer.	*/

	Location = ViewCam->GetLocation ();

	/*	Transform the ground
		plane into the view.	*/

	MoveBy.v [0] = fmod ( Location.v [0], ( 6000.0 / GRIDSIZE ) );
	MoveBy.v [1] = Location.v [1];
	MoveBy.v [2] = fmod ( Location.v [2], ( 6000.0 / GRIDSIZE ) );

	ViewCam->GetMatrix ().Multiply ( ( GRIDSIZE * GRIDSIZE ), ( POINT * ) Vertices, ViewVerts, MoveBy );

	/*	Ground is visible,
		apply perspective.	*/

	Buffer.SetTexture ( &MyTexture, FALSE );
	Buffer.SetWindow ( View.GetWindow () );
	for ( i = 0; i < ( GRIDSIZE - 1 ); i++ ) {
		for ( j = 0; j < ( GRIDSIZE - 1 ); j++ ) {
			nVerts = 4;
			InPoly [0] = ViewVerts [i + ( GRIDSIZE * j )];
			InPoly [1] = ViewVerts [i + ( GRIDSIZE * j ) + 1];
			InPoly [2] = ViewVerts [i + ( GRIDSIZE * ( j + 1 ) ) + 1];
			InPoly [3] = ViewVerts [i + ( GRIDSIZE * ( j + 1 ) ) ];
			if ( View.ClipToViewFrustum ( &nVerts, InPoly, OutPoly, InTex, OutTex ) == TRUE ) {
				View.Project ( nVerts, OutPoly, IntPoly );
				Colour = cLight->GetLightIntensity ( &InPoly [0], &InPoly [1], &InPoly [2], 112 );
				Buffer.TextureMapPolygon ( nVerts, IntPoly, OutTex, TRUE, Colour );
			}
		}
	}
}




















/*
CODE FOR THE DOT PLANE
			} /*else {
				if ( ViewVerts [i+(GRIDSIZE*j)].v [2] > 0.0 ) {
					Window.Normalise ( 1, &ViewVerts [i+(GRIDSIZE*j)], &Integer );
					if ( Integer.x > 0 && Integer.x < 320 && Integer.y > 0.0 && Integer.y < 400.0 ) {
						Buffer.PUTPIXEL ( Integer.x, Integer.y, 100 );
					}
				}
			}   */
  */
