
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	MODETEX.CPP                                                         //
//                                                                          //
//	Implementation of MODE class.                                       //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////
	

#include "MODE\MODE.H"

#include <conio.h>
#include <stdio.h>
#include <math.h>
#include <dos.h>

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// This describes the clipping window for the purposes of Abrash's funcs.   //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

int ClipMinX = 0, ClipMinY= 0, ClipMaxX = 399, ClipMaxY = 319;

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// These are fixed point macros for Abrash's texture mapper.                //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

#define INT_TO_FIXED(a)	(long)(((long)(a))<<16)
#define ROUND_FIXED_TO_INT(FixedVal) ( ( int ) ( ( FixedVal + 0x8000 ) >> 16) )

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// These are statics to avoid excessive parameter passing on the stack.     //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

static int	MaxVert, DestY, nVertices, TexMapDeltaY;
static PIXEL	tColour;
static INTPOINT	*VertexPtr;
static INTPOINT	*TexVertsPtr;
static PIXEL	*TexMapBits;

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// This function is Abrash's edge stepper for the edges of the polygon.     //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

int GRAPHICSMODE :: StepEdge ( EDGESCAN * Edge )

/*	PRE	:
	POST	:
*/

{
	if ( --Edge->RemainingScans == 0 ) {
		if ( SetUpEdge ( Edge, Edge->CurrentEnd ) == 0 ) {
			return ( 0 );
		}
		return ( 1 );
	}

	Edge->SourceX += Edge->SourceStepX;
	Edge->SourceY += Edge->SourceStepY;

	Edge->DestX += Edge->DestXIntStep;  /* whole pixel step */

	if ( ( Edge->DestXErrTerm += Edge->DestXAdjUp ) > 0 ) {
		Edge->DestX += Edge->DestXDirection;
		Edge->DestXErrTerm -= Edge->DestXAdjDown;
	}

	return ( 1 );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// This function is Abrash's edge stepper setup for polygon edges.          //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

int GRAPHICSMODE :: SetUpEdge ( EDGESCAN * Edge, int StartVert )

/*	PRE	:
	POST	:
*/

{
	int NextVert, DestXWidth;
	long DestYHeight;

	for (;;) {
		if ( StartVert == MaxVert ) {
			return ( 0 );
		}

		NextVert = StartVert + Edge->Direction;
		if ( NextVert >= nVertices ) {
			NextVert = 0;
		} else if ( NextVert < 0 ) {
			NextVert = nVertices - 1;
		}

		if ( ( Edge->RemainingScans = VertexPtr [NextVert].y - VertexPtr [StartVert].y) != 0 ) {
			DestYHeight = INT_TO_FIXED ( Edge->RemainingScans );
			Edge->CurrentEnd = NextVert;
			Edge->SourceX = INT_TO_FIXED ( TexVertsPtr [StartVert].x );
			Edge->SourceY = INT_TO_FIXED ( TexVertsPtr [StartVert].y );
			Edge->SourceStepX = FixedDiv ( INT_TO_FIXED ( TexVertsPtr [NextVert].x ) - Edge->SourceX, DestYHeight );
			Edge->SourceStepY = FixedDiv(INT_TO_FIXED( TexVertsPtr [NextVert].y ) - Edge->SourceY, DestYHeight );
			Edge->DestX = VertexPtr [StartVert].x;
			if ( ( DestXWidth = ( VertexPtr [NextVert].x - VertexPtr [StartVert].x ) ) < 0 ) {
				Edge->DestXDirection = -1;
				DestXWidth = -DestXWidth;
				Edge->DestXErrTerm = 1 - Edge->RemainingScans;
				Edge->DestXIntStep = -( DestXWidth / Edge->RemainingScans );
			} else {
				Edge->DestXDirection = 1;
				Edge->DestXErrTerm = 0;
				Edge->DestXIntStep = DestXWidth / Edge->RemainingScans;
			}
			Edge->DestXAdjUp = DestXWidth % Edge->RemainingScans;
			Edge->DestXAdjDown = Edge->RemainingScans;
			return ( 1 );
		}
		StartVert = NextVert;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Abrash's outline scanner for the stepped edges of a polygon.             //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void GRAPHICSMODE :: ScanOutLine ( EDGESCAN * LeftEdge, EDGESCAN * RightEdge )

/*	PRE	:
	POST	:
*/

{
	long		SourceX = LeftEdge->SourceX;
	long		SourceY = LeftEdge->SourceY;
	int		DestX = LeftEdge->DestX;
	int		DestXMax = RightEdge->DestX;
	long   		DestWidth;
	long		SourceXStep, SourceYStep;

	if ( ( DestXMax <= ClipMinX ) || ( DestX >= ClipMaxX ) ) {
		return;
	}

	if ( ( DestXMax - DestX ) <= 0 ) {
		return;
	}

	DestWidth = INT_TO_FIXED ( DestXMax - DestX );

	SourceXStep = FixedDiv ( RightEdge->SourceX - SourceX, DestWidth );
	SourceYStep = FixedDiv ( RightEdge->SourceY - SourceY, DestWidth );
	SourceX += SourceXStep >> 1;
	SourceY += SourceYStep >> 1;

	if ( DestXMax > ClipMaxX ) {
		DestXMax = ClipMaxX;
	}

	if ( DestX < ClipMinX ) {
		SourceX += FixedMul ( SourceXStep, INT_TO_FIXED ( ClipMinX - DestX ) );
		SourceY += FixedMul ( SourceYStep, INT_TO_FIXED ( ClipMinX - DestX ) );
		DestX = ClipMinX;
	}

	int Masks [] = { 1, 2, 4, 8 };

	MAPMASK ( Masks [DestY&3] );
		{
			PIXEL *Ptr, Pix;

			Ptr = GETADDRESS ( DestY, DestX );
			SourceX += 0x8000;
			SourceY += 0x8000;
			for (; DestX < DestXMax; DestX++ ) {
				Pix = * ( TexMapBits + ( int ) ( SourceX >> 16 ) + ( ( int ) ( SourceY >> 16 ) << TexMapDeltaY ) );
				*Ptr = ( Pix + tColour );
				SourceX += SourceXStep;
				SourceY += SourceYStep;
				Ptr += 80;
			}
		}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Render a texture mapped polygon onto the screen.                         //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void GRAPHICSMODE :: RenderPolygon ( int nVerts, INTPOINT *Verts, INTPOINT *tVerts, TEXTUREMAP *Texture, BOOL Culled, PIXEL Colour )

/*	PRE	:	TRUE
	POST	:	Render textured polygon with the given colour.
*/

{
	int		MinY, MaxY, MinVert, i, j;
	EDGESCAN	LeftEdge, RightEdge;
	static INTPOINT MyVerts [64], MyTexVerts [64];

	tColour = Colour;
	nVertices = nVerts;
	VertexPtr = Verts;
	TexVertsPtr = tVerts;
	TexMapBits = Texture->GetPixels ();
	TexMapDeltaY = Texture->GetDelta ();

	if (nVertices < 3) {
		return;
	}

	if ( Culled == TRUE ) {

		/*	Culled, so vertices
			are in reverse order.	*/

		for ( j = 0, i = ( nVertices - 1 ); i > -1; i--, j++ ) {
			MyVerts [i].x = VertexPtr[j].y;
			MyVerts [i].y = VertexPtr[j].x;
			MyTexVerts [i].x = TexVertsPtr[j].x;
			MyTexVerts [i].y = TexVertsPtr[j].y;
		}
	} else {

		/*	Was not culled so vertices
			remain in the same order.	*/

		for ( i = 0; i < nVertices; i++ ) {
			MyVerts [i].x = VertexPtr[i].y;
			MyVerts [i].y = VertexPtr[i].x;
			MyTexVerts [i].x = TexVertsPtr[i].x;
			MyTexVerts [i].y = TexVertsPtr[i].y;
		}
	}

	TexVertsPtr = MyTexVerts;
        VertexPtr = MyVerts;

	MinY = 32767;
	MaxY = -32767;
	for (i = 0; i < nVertices; i++ ) {
		if ( VertexPtr [i].y < MinY ) {
			MinY = VertexPtr [i].y;
			MinVert = i;
		}
		if ( VertexPtr [i].y > MaxY ) {
			MaxY = VertexPtr [i].y;
			MaxVert = i;
		}
	}

	if ( MinY >= MaxY ) {
		return;
	}
	DestY = MinY;

	LeftEdge.Direction = -1;
	SetUpEdge ( &LeftEdge, MinVert );
	RightEdge.Direction = 1;
	SetUpEdge ( &RightEdge, MinVert );

	for (;;) {
		if ( DestY >= ClipMaxY ) {
			return;
		}
		if ( DestY >= ClipMinY ) {
			ScanOutLine ( &LeftEdge, &RightEdge );
		}
		if ( !StepEdge ( &LeftEdge ) ) {
			break;
		}
		if ( !StepEdge ( &RightEdge ) ) {
			break;
		}
		DestY++;
	}
}
